@@ -1,6 +1,7 @@
 """Helper functions to make data handling and conversions easier
 
 # Figures 0.3.13 - Defining scope of this module
+# Figures 0.4.x - Yep, this module is still handy and the scope hasn't exploded
 
 The purpose of this module is to provide conveniece methods around commonly
 executed statements. These convenience methods should serve as shorthand for
@@ -34,6 +35,8 @@ repeat yourself) the code and make the code more readable
 
 ## What does not belong here?
 
+* Most importantly, if you have to import from another figures module, it does
+  not belong here!
 * "Feature" functionality does not belong here
 * Long functions do not belong here
 * Code that communicates outside of Figures does not belong here. No database,
@@ -42,19 +45,24 @@ repeat yourself) the code and make the code more readable
 This is not an exhaustive list. We'll grow it as needed.
 
 An important point is that we should not expect this module to be a permanent
-home for functionality.
+home for the functionality included. As we evolve Figures, we may find functions
+here that have a stronger context with another module. For example, we've got
+a decent set of date oriented functions that are candidatdes for a datetime and
+date handling module.
 """
 
+from __future__ import absolute_import
 import calendar
 import datetime
+from importlib import import_module
 from django.conf import settings
 from django.utils.timezone import utc
 
 from dateutil.parser import parse as dateutil_parse
 from dateutil.relativedelta import relativedelta
-from dateutil.rrule import rrule, MONTHLY
 
 from opaque_keys.edx.keys import CourseKey
+import six
 
 
 def is_multisite():
@@ -79,6 +87,23 @@ def log_pipeline_errors_to_db():
     return bool(settings.FEATURES.get('FIGURES_LOG_PIPELINE_ERRORS_TO_DB', True))
 
 
+def import_from_path(path):
+    """
+    Import a function or class from a its string Python path.
+
+    Note: This help does _not_ attempt to handle exceptions well.
+      Instead it throws them as is. The rationale is that such exceptions are
+      only fixable at the deploy time and attempting to handle such errors
+      would risk hiding the errors and making it more difficult to fix.
+
+    :param path: string path in the format "module.submodule:variable".
+    :return object
+    """
+    module_path, variable_name = path.split(':', 1)
+    module = import_module(module_path)
+    return getattr(module, variable_name)
+
+
 def as_course_key(course_id):
     """Returns course id as a CourseKey instance
 
@@ -92,7 +117,7 @@ def as_course_key(course_id):
     """
     if isinstance(course_id, CourseKey):
         return course_id
-    elif isinstance(course_id, basestring):  # noqa: F821
+    elif isinstance(course_id, six.string_types):  # noqa: F821
         return CourseKey.from_string(course_id)
     else:
         raise TypeError('Unable to convert course id with type "{}"'.format(
@@ -117,9 +142,9 @@ def as_datetime(val):
             year=val.year,
             month=val.month,
             day=val.day,
-            ).replace(tzinfo=utc)
+        ).replace(tzinfo=utc)
 
-    elif isinstance(val, basestring):  # noqa: F821
+    elif isinstance(val, six.string_types):  # noqa: F821
         return dateutil_parse(val).replace(tzinfo=utc)
     else:
         raise TypeError(
@@ -143,7 +168,7 @@ def as_date(val):
         return val.date()
     elif isinstance(val, datetime.date):
         return val
-    elif isinstance(val, basestring):  # noqa: F821
+    elif isinstance(val, six.string_types):  # noqa: F821
         return dateutil_parse(val).date()
     else:
         raise TypeError(
@@ -169,14 +194,29 @@ def prev_day(val):
     return days_from(val, -1)
 
 
+def utc_yesterday():
+    """Get "yesterday" form the utc datetime
+
+    We primarily use this for the daily metrics collection. However, it proves
+    handy as a convenience function for exploring data in the Django shell
+    """
+    return prev_day(datetime.datetime.utcnow().date())
+
+
 def days_in_month(month_for):
     _, num_days_in_month = calendar.monthrange(month_for.year, month_for.month)
     return num_days_in_month
 
 
+def is_past_date(val):
+    return as_date(val) < datetime.date.today()
+
+
 # TODO: Consider changing name to 'months_back_iterator' or similar
 def previous_months_iterator(month_for, months_back):
-    """Iterator returns a year,month tuple for n months including the month_for
+    """Iterator returns a year,month tuple for n months including the month_for.
+    months_back is a misnomer as iteration includes the start month.  The actual
+    number of previous months iterated is months_back minus one.
 
     month_for is either a date, datetime, or tuple with year and month
     months back is the number of months to iterate
@@ -186,9 +226,10 @@ def previous_months_iterator(month_for, months_back):
         # TODO make sure we've got just two values in the tuple
         month_for = datetime.date(year=month_for[0], month=month_for[1], day=1)
     if isinstance(month_for, (datetime.datetime, datetime.date)):
-        start_month = month_for - relativedelta(months=months_back)
+        start_month = month_for - relativedelta(months=(max(0, months_back - 1)))
 
-    for dt in rrule(freq=MONTHLY, dtstart=start_month, until=month_for):
+    for n_months in range(max(1, months_back)):
+        dt = start_month + relativedelta(months=n_months)
         last_day_of_month = days_in_month(month_for=dt)
         yield (dt.year, dt.month, last_day_of_month)
 
