@@ -3,7 +3,9 @@
 TODO: Create a base "SiteModel" or a "SiteModelMixin"
 """
 
+from __future__ import absolute_import
 from datetime import date
+from time import time
 from django.conf import settings
 from django.contrib.sites.models import Site
 from django.core.validators import MaxValueValidator, MinValueValidator
@@ -15,6 +17,10 @@ from jsonfield import JSONField
 
 from model_utils.models import TimeStampedModel
 
+from figures.compat import CourseEnrollment
+from figures.helpers import as_course_key, utc_yesterday
+from figures.progress import EnrollmentProgress
+
 
 def default_site():
     """
@@ -33,14 +39,15 @@ class CourseDailyMetrics(TimeStampedModel):
     adding a SiteDailyMetrics foreign key. This is subject to change as the code
     evolves.
     """
-    site = models.ForeignKey(Site)
-    date_for = models.DateField()
+    # TODO: Review the most appropriate on_delete behaviour
+    site = models.ForeignKey(Site, on_delete=models.CASCADE)
+    date_for = models.DateField(db_index=True)
 
     # Leaving as a simple string for initial development
     # TODO: Follow on to decide if we want to make this an FK to
     # the CourseOverview model or have the course_id be a
     # CourseKeyField
-    course_id = models.CharField(max_length=255)
+    course_id = models.CharField(max_length=255, db_index=True)
     enrollment_count = models.IntegerField()
     active_learners_today = models.IntegerField()
     # Do we want cumulative average progress for the month?
@@ -94,10 +101,10 @@ class SiteDailyMetrics(TimeStampedModel):
     When we upgrade to MAU 2G, we'll add a new field, 'active_users_today'
     and pull the data from the previous day's live active user capture
     """
-
-    site = models.ForeignKey(Site)
+    # TODO: Review the most appropriate on_delete behaviour
+    site = models.ForeignKey(Site, on_delete=models.CASCADE)
     # Date for which this record's data are collected
-    date_for = models.DateField()
+    date_for = models.DateField(db_index=True)
 
     # Under review for deletion
     cumulative_active_user_count = models.IntegerField(blank=True, null=True)
@@ -110,6 +117,8 @@ class SiteDailyMetrics(TimeStampedModel):
     # Should change this to default value of 0
     mau = models.IntegerField(blank=True, null=True)
 
+    # TODO: Add field for number of CDMs reported
+
     class Meta:
         """
         SiteDailyMetrics view and serializer tests fail when we include 'site'
@@ -154,8 +163,8 @@ class SiteMonthlyMetrics(TimeStampedModel):
 
 
     """
-
-    site = models.ForeignKey(Site)
+    # TODO: Review the most appropriate on_delete behaviour
+    site = models.ForeignKey(Site, on_delete=models.CASCADE)
     # Month for which this record's data are collected
     # Important fields are year and month
     month_for = models.DateField()
@@ -188,15 +197,221 @@ class SiteMonthlyMetrics(TimeStampedModel):
                                                            defaults=defaults)
 
 
-class LearnerCourseGradeMetricsManager(models.Manager):
-    """Custom model manager for LearnerCourseGrades model
+class EnrollmentDataManager(models.Manager):
+    """Custom model manager for EnrollmentData
+
+    Initial purpose is to handle the logic of creating and updating
+    EnrollmentData instances.
+
     """
-    def most_recent_for_learner_course(self, user, course_id):
-        queryset = self.filter(user=user, course_id=str(course_id))
-        if queryset:
-            return queryset.order_by('-date_for')[0]
+    def set_enrollment_data(self, site, user, course_id, course_enrollment=None):
+        """
+        This is an expensive call as it needs to call CourseGradeFactory if
+        there is not already a LearnerCourseGradeMetrics record for the learner
+        """
+        if not course_enrollment:
+            # For now, let it raise a `CourseEnrollment.DoesNotExist
+            # Later on we can add a try block and raise out own custom
+            # exception
+            course_enrollment = CourseEnrollment.objects.get(
+                user=user,
+                course_id=as_course_key(course_id))
+
+        defaults = dict(
+            is_enrolled=course_enrollment.is_active,
+            date_enrolled=course_enrollment.created,
+        )
+
+        # Note: doesn't use site for filtering
+        lcgm = LearnerCourseGradeMetrics.objects.latest_lcgm(
+            user=user,
+            course_id=str(course_id))
+        if lcgm:
+            # do we already have an enrollment data record
+            # We may change this to use
+            progress_data = dict(
+                date_for=lcgm.date_for,
+                is_completed=lcgm.completed,
+                progress_percent=lcgm.progress_percent,
+                points_possible=lcgm.points_possible,
+                points_earned=lcgm.points_earned,
+                sections_possible=lcgm.sections_possible,
+                sections_worked=lcgm.sections_worked
+            )
         else:
-            return None
+            ep = EnrollmentProgress(user=user, course_id=course_id)
+            # TODO: If we get progress worked and there is no LCGM, then we have
+            # a bug OR there was progress after the last daily metrics collection
+            progress_data = dict(
+                date_for=date.today(),
+                is_completed=ep.is_completed(),
+                progress_percent=ep.progress_percent(),
+                points_possible=ep.progress.get('points_possible', 0),
+                points_earned=ep.progress.get('points_earned', 0),
+                sections_possible=ep.progress.get('sections_possible', 0),
+                sections_worked=ep.progress.get('sections_worked', 0)
+            )
+        defaults.update(progress_data)
+
+        obj, created = self.update_or_create(
+            site=site,
+            user=user,
+            course_id=str(course_id),
+            defaults=defaults)
+        return obj, created
+
+    def update_metrics(self, site, course_enrollment, force_update=False):
+        """
+        This is an expensive call as it needs to call CourseGradeFactory if
+        there is not already a LearnerCourseGradeMetrics record for the learner
+
+        """
+        date_for = utc_yesterday()
+
+        # check if we already have a record for the date for EnrollmentData
+
+        # Alternately, we could use a try/except on a 'get' call, however, this
+        # would be much slower for a bunch of new enrollments
+
+        # Should only be one even if we don't include the site in the query
+        # because course_id should be globally unique
+        # If course id is ever NOT globally unique, then we need to add site
+        # to the query
+        ed_recs = EnrollmentData.objects.filter(
+            user_id=course_enrollment.user_id,
+            course_id=str(course_enrollment.course_id))
+
+        if not ed_recs or ed_recs[0].date_for < date_for or force_update:
+            # We do the update
+            start_time = time()
+            # get the progress data
+            ep = EnrollmentProgress(user=course_enrollment.user,
+                                    course_id=str(course_enrollment.course_id))
+            defaults = dict(
+                date_for=date_for,
+                is_completed=ep.is_completed(),
+                progress_percent=ep.progress_percent(),
+                points_possible=ep.progress.get('points_possible', 0),
+                points_earned=ep.progress.get('points_earned', 0),
+                sections_possible=ep.progress.get('sections_possible', 0),
+                sections_worked=ep.progress.get('sections_worked', 0),
+                is_enrolled=course_enrollment.is_active,
+                date_enrolled=course_enrollment.created,
+            )
+            elapsed = time() - start_time
+            defaults['collect_elapsed'] = elapsed
+
+            ed_rec, created = self.update_or_create(
+                site=site,
+                user=course_enrollment.user,
+                course_id=str(course_enrollment.course_id),
+                defaults=defaults)
+            # create a new LCGM record
+            # if it already exists for the day
+            LearnerCourseGradeMetrics.objects.update_or_create(
+                site=ed_rec.site,
+                user=ed_rec.user,
+                course_id=ed_rec.course_id,
+                date_for=date_for,
+                defaults=dict(
+                    points_possible=ed_rec.points_possible,
+                    points_earned=ed_rec.points_earned,
+                    sections_worked=ed_rec.sections_worked,
+                    sections_possible=ed_rec.sections_possible,
+                    collect_elapsed=elapsed
+                )
+            )
+            return ed_rec, created
+        else:
+            return ed_recs[0], False
+
+
+@python_2_unicode_compatible
+class EnrollmentData(TimeStampedModel):
+    """Tracks most recent enrollment data for an enrollment
+
+    An enrollment is a unique site + user + course
+
+    This model stores basic enrollment information and latest progress
+    The purpose of this class is for query performance for the 'learner-metrics'
+    API endpoint which is needed for the learner progress overview page.
+
+    This is an intial take on caching current enrollment data with the dual
+    purposes of speeding up the learner-metrics endpoint needed for the LPO page
+    as well as doing so in clear maintainable code.
+
+    At some point in the future, we'll probably have to construct a key-value
+    high performance storage, but for now, we'd like to see how far we can get
+    with the basic Django architecture. Plus this simplifies running Figures on
+    small Open edX LMS deployments
+    """
+    site = models.ForeignKey(Site, on_delete=models.CASCADE)
+    user = models.ForeignKey(settings.AUTH_USER_MODEL,
+                             on_delete=models.CASCADE)
+    course_id = models.CharField(max_length=255, db_index=True)
+    date_for = models.DateField(db_index=True)
+
+    # Date enrolled is from CourseEnrollment.created
+    date_enrolled = models.DateField(db_index=True)
+
+    # From CourseEnrollment.is_active
+    is_enrolled = models.BooleanField()
+
+    # From LCGM.completed property methods
+    is_completed = models.BooleanField()
+    progress_percent = models.FloatField(default=0.00)
+
+    # from LCGM fields
+    points_possible = models.FloatField()
+    points_earned = models.FloatField()
+    sections_worked = models.IntegerField()
+    sections_possible = models.IntegerField()
+
+    # seconds it took to collect progress data
+    collect_elapsed = models.FloatField(null=True)
+
+    objects = EnrollmentDataManager()
+
+    class Meta:
+        unique_together = ('site', 'user', 'course_id')
+
+    def __str__(self):
+        return '{} {} {} {}'.format(
+            self.id, self.site.domain, self.user.email, self.course_id)
+
+    @property
+    def progress_details(self):
+        """This method gets the progress details
+        This method is a temporary fix until the serializers are updated.
+        """
+        return dict(
+            points_possible=self.points_possible,
+            points_earned=self.points_earned,
+            sections_worked=self.sections_worked,
+            sections_possible=self.sections_possible,
+        )
+
+
+class LearnerCourseGradeMetricsManager(models.Manager):
+    """Custom model manager for LearnerCourseGradeMetrics model
+    """
+    def latest_lcgm(self, user, course_id):
+        """Gets the most recent record for the given user and course
+
+        We have this because we implement sparse data, meaning we only create
+        new records when data has changed. this means that for a given course,
+        learners may not have the same "most recent date"
+
+        This means we have to be careful of where we use this method in our
+        API as it costs a query per call. We will likely require restructuring
+        or augmenting our data if we need to bulk retrieve
+
+        TODO: Consider if we want to add 'site' as a parameter and update the
+        uniqueness constraint to be: site, course_id, user, date_for
+        """
+        queryset = self.filter(user=user,
+                               course_id=str(course_id)).order_by('-date_for')
+        return queryset[0] if queryset else None
 
     def most_recent_for_course(self, course_id):
         statement = """ \
@@ -268,16 +483,16 @@ class LearnerCourseGradeMetrics(TimeStampedModel):
     Purpose is primarliy to improve performance for the front end. In addition,
     data collected can be used for course progress over time
 
-    We're capturing data from figures.metrics.LearnerCourseGrades
+    We're capturing data from figures.progress.EnrollmentProgress
 
-    Note: We're probably going to move ``LearnerCourseGrades`` to figures.pipeline
+    Note: We're probably going to move ``EnrollmentProgress`` to figures.pipeline
     since that class will only be needed by the pipeline
 
     Even though this is for a course enrollment, we're mapping to the user
     and providing course id instead of an FK relationship to the courseenrollment
-    Reason is we're likely more interested in the learner and the course than the specific
-    course enrollment. Also means that the Figures models do not have a hard
-    dependency on models in edx-platform
+    Reason is we're likely more interested in the learner and the course than
+    the specific course enrollment. Also means that the Figures models do not
+    have a hard dependency on models in edx-platform
 
     Considered using DecimalField for points as we can control the decimal places
     But for now, using float, as I'm not entirely sure how many decimal places are
@@ -287,19 +502,28 @@ class LearnerCourseGradeMetrics(TimeStampedModel):
     TODO: Add fields
         `is_active` - get the 'is_active' value from the enrollment at the time
         this record is created
-        `completed` - This lets us filter on a table column instead of calculating it
+        `completed` - This lets us filter on a table column instead of
+                      calculating it
     TODO: Add index on 'course_id', 'date_for', 'completed'
     """
-    site = models.ForeignKey(Site)
-    date_for = models.DateField()
+    # TODO: Review the most appropriate on_delete behaviour
+    site = models.ForeignKey(Site, on_delete=models.CASCADE)
+    date_for = models.DateField(db_index=True)
     # TODO: We should require the user
-    user = models.ForeignKey(settings.AUTH_USER_MODEL, blank=True, null=True)
-    course_id = models.CharField(max_length=255, blank=True)
+    # TODO: Review the most appropriate on_delete behaviour
+    user = models.ForeignKey(settings.AUTH_USER_MODEL,
+                             blank=True,
+                             null=True,
+                             on_delete=models.CASCADE)
+    course_id = models.CharField(max_length=255, blank=True, db_index=True)
     points_possible = models.FloatField()
     points_earned = models.FloatField()
     sections_worked = models.IntegerField()
     sections_possible = models.IntegerField()
 
+    # seconds it took to collect progress data
+    collect_elapsed = models.FloatField(null=True)
+
     objects = LearnerCourseGradeMetricsManager()
 
     class Meta:
@@ -342,7 +566,8 @@ class LearnerCourseGradeMetrics(TimeStampedModel):
 
     @property
     def completed(self):
-        return self.sections_worked > 0 and self.sections_worked == self.sections_possible
+        return (self.sections_worked > 0 and
+                self.sections_worked == self.sections_possible)
 
 
 @python_2_unicode_compatible
@@ -368,8 +593,14 @@ class PipelineError(TimeStampedModel):
     error_data = JSONField()
     # Attributes for convenient querying
     course_id = models.CharField(max_length=255, blank=True)
-    user = models.ForeignKey(settings.AUTH_USER_MODEL, blank=True, null=True)
-    site = models.ForeignKey(Site, blank=True, null=True)
+    # TODO: Review the most appropriate on_delete behaviour
+    user = models.ForeignKey(settings.AUTH_USER_MODEL,
+                             blank=True,
+                             null=True,
+                             on_delete=models.CASCADE)
+    site = models.ForeignKey(Site, blank=True,
+                             null=True,
+                             on_delete=models.CASCADE)
 
     class Meta:
         ordering = ['-created']
@@ -379,7 +610,8 @@ class PipelineError(TimeStampedModel):
 
 
 class BaseDateMetricsModel(TimeStampedModel):
-    site = models.ForeignKey(Site, default=default_site)
+    # TODO: Review the most appropriate on_delete behaviour
+    site = models.ForeignKey(Site, default=default_site, on_delete=models.CASCADE)
     date_for = models.DateField()
 
     class Meta:
@@ -402,7 +634,9 @@ class SiteMauMetricsManager(models.Manager):
         """Return the latest record for the given site, month, and year
         If no record found, returns 'None'
         """
-        queryset = self.filter(site=site, date_for__year=year, date_for__month=month)
+        queryset = self.filter(site=site,
+                               date_for__year=year,
+                               date_for__month=month)
         return queryset.order_by('-modified').first()
 
 
