@@ -11,27 +11,32 @@ Future: add a remote mode to pull data via REST API
 
 # TODO: Move extractors to figures.pipeline.extract module
 """
-import datetime
+from __future__ import absolute_import
 from decimal import Decimal
 import logging
 
+from dateutil.relativedelta import relativedelta
 from django.db import transaction
-from django.utils.timezone import utc
 
-from courseware.models import StudentModule  # pylint: disable=import-error
-from openedx.core.djangoapps.content.course_overviews.models import CourseOverview  # noqa pylint: disable=import-error
-from student.models import CourseEnrollment  # pylint: disable=import-error
 from student.roles import CourseCcxCoachRole, CourseInstructorRole, CourseStaffRole  # noqa pylint: disable=import-error
 
-from figures.helpers import as_course_key, as_datetime, next_day, prev_day
+from figures.compat import (CourseEnrollment,
+                            CourseOverview,
+                            GeneratedCertificate,
+                            StudentModule)
+from figures.helpers import as_course_key, as_datetime, is_past_date, next_day
 import figures.metrics
 from figures.models import CourseDailyMetrics, PipelineError
 from figures.pipeline.logger import log_error
 import figures.pipeline.loaders
 from figures.pipeline.enrollment_metrics import bulk_calculate_course_progress_data
+from figures.pipeline.enrollment_metrics_next import (
+    calculate_course_progress as calculate_course_progress_next
+)
+
 from figures.serializers import CourseIndexSerializer
-from figures.compat import GeneratedCertificate
 import figures.sites
+from figures.pipeline.helpers import pipeline_date_for_rule
 
 
 logger = logging.getLogger(__name__)
@@ -56,7 +61,7 @@ def get_enrolled_in_exclude_admins(course_id, date_for=None):
     staff = CourseStaffRole(course_locator).users_with_role()
     admins = CourseInstructorRole(course_locator).users_with_role()
     coaches = CourseCcxCoachRole(course_locator).users_with_role()
-    filter_args = dict(course_id=course_id, is_active=1)
+    filter_args = dict(course_id=course_locator, is_active=1)
 
     if date_for:
         filter_args.update(dict(created__lt=as_datetime(next_day(date_for))))
@@ -162,7 +167,16 @@ def get_days_to_complete(course_id, date_for):
                      course_id=course_id,
                      user_id=cert.user.id,
                      ))
-        days.append((cert.created_date - ce[0].created).days)
+        try:
+            days.append((cert.created_date - ce[0].created).days)
+        except IndexError:
+            # sometimes a course enrollment is deleted after the cert is generated.  why, who knows?
+            # in which case just leave out that data
+            errors.append(
+                dict(msg='No CourseEnrollment matching user course certificate',
+                     course_id=course_id,
+                     user_id=cert.user.id,
+                     ))
     return dict(days=days, errors=errors)
 
 
@@ -225,26 +239,36 @@ class CourseDailyMetricsExtractor(object):
     BUT, we will then need to find a transform
     """
 
-    def extract(self, course_id, date_for=None, **_kwargs):
-        """
-            defaults = dict(
+    def extract(self, course_id, date_for, ed_next=False, **_kwargs):
+        """Extracts (collects) aggregated course level data
+
+        Args:
+            course_id (:obj:`str` or :obj:`CourseKey`): The course for which we collect data
+            date_for (str or date): Deprecated. Was to backfill data.
+                Specialized TBD backfill data will be called instead.
+            ed_next (bool, optional): "Enrollment Data Next" flag. If set to `True`
+                then we collect metrics with our updated workflow. See here:
+                https://github.com/appsembler/figures/issues/428
+
+        Returns:
+            dict with aggregate course level metrics
+
+            ```
+            dict(
                 enrollment_count=data['enrollment_count'],
                 active_learners_today=data['active_learners_today'],
                 average_progress=data.get('average_progress', None),
                 average_days_to_complete=data.get('average_days_to_complete, None'),
                 num_learners_completed=data['num_learners_completed'],
             )
-        TODO: refactor this class
+            ```
+
+        TODO: refactor this class. It doesn't need to be a class. Can be a
+        standalone function
         Add lazy loading method to load course enrollments
         - Create a method for each metric field
         """
 
-        # Update args if not assigned
-        if not date_for:
-            date_for = prev_day(
-                datetime.datetime.utcnow().replace(tzinfo=utc).date()
-            )
-
         # We can turn this series of calls into a parallel
         # set of calls defined in a ruleset instead of hardcoded here after
         # retrieving the core quersets
@@ -269,9 +293,48 @@ class CourseDailyMetricsExtractor(object):
         data['active_learners_today'] = active_learners_today
 
         # Average progress
-        progress_data = bulk_calculate_course_progress_data(course_id=course_id,
-                                                            date_for=date_for)
-        data['average_progress'] = progress_data['average_progress']
+        # Progress data cannot be reliable for backfills or for any date prior to yesterday
+        # without using StudentModuleHistory so we skip getting this data if running
+        # for a day earlier than previous day (i.e., not during daily update of CDMs),
+        #  especially since it is so expensive to calculate.
+        # Note that Avg() applied across null and decimal vals for aggregate average_progress
+        # will correctly ignore nulls
+        # TODO: Reconsider this if we implement either StudentModuleHistory-based queries
+        # (if so, you will need to add any types you want to
+        # StudentModuleHistory.HISTORY_SAVING_TYPES)
+        # TODO: Reconsider this once we switch to using Persistent Grades
+        if is_past_date(date_for + relativedelta(days=1)):  # more than 1 day in past
+            data['average_progress'] = None
+            msg = ('FIGURES:PIPELINE:CDM Declining to calculate average progress for a past date'
+                   ' date_for={date_for}, course_id="{course_id}"')
+            logger.debug(msg.format(date_for=date_for, course_id=course_id))
+        else:
+            try:
+                # This conditional check is an interim solution until we make
+                # the progress function configurable and able to run Figures
+                # plugins
+                if ed_next:
+                    progress_data = calculate_course_progress_next(course_id=course_id)
+                else:
+                    progress_data = bulk_calculate_course_progress_data(course_id=course_id,
+                                                                        date_for=date_for)
+                data['average_progress'] = progress_data['average_progress']
+            except Exception:  # pylint: disable=broad-except
+                # Broad exception for starters. Refine as we see what gets caught
+                # Make sure we set the average_progres to None so that upstream
+                # does not think things are normal
+                data['average_progress'] = None
+
+                if ed_next:
+                    prog_func = 'calculate_course_progress_next'
+                else:
+                    prog_func = 'bulk_calculate_course_progress_data'
+
+                msg = ('FIGURES:FAIL {prog_func}'
+                       ' date_for={date_for}, course_id="{course_id}"')
+                logger.exception(msg.format(prog_func=prog_func,
+                                            date_for=date_for,
+                                            course_id=course_id))
 
         data['average_days_to_complete'] = get_average_days_to_complete(
             course_id, date_for,)
@@ -290,10 +353,11 @@ class CourseDailyMetricsLoader(object):
         self.extractor = CourseDailyMetricsExtractor()
         self.site = figures.sites.get_site_for_course(self.course_id)
 
-    def get_data(self, date_for):
+    def get_data(self, date_for, ed_next=False):
         return self.extractor.extract(
             course_id=self.course_id,
-            date_for=date_for)
+            date_for=date_for,
+            ed_next=ed_next)
 
     @transaction.atomic
     def save_metrics(self, date_for, data):
@@ -303,23 +367,25 @@ class CourseDailyMetricsLoader(object):
         Raises django.core.exceptions.ValidationError if the record fails
         validation
         """
+        defaults = dict(
+            enrollment_count=data['enrollment_count'],
+            active_learners_today=data['active_learners_today'],
+            average_days_to_complete=int(round(data['average_days_to_complete'])),
+            num_learners_completed=data['num_learners_completed'],
+        )
+        if data['average_progress'] is not None:
+            defaults['average_progress'] = str(data['average_progress'])
 
         cdm, created = CourseDailyMetrics.objects.update_or_create(
-            course_id=self.course_id,
+            course_id=str(self.course_id),
             site=self.site,
             date_for=date_for,
-            defaults=dict(
-                enrollment_count=data['enrollment_count'],
-                active_learners_today=data['active_learners_today'],
-                average_progress=str(data['average_progress']),
-                average_days_to_complete=int(round(data['average_days_to_complete'])),
-                num_learners_completed=data['num_learners_completed'],
-            )
+            defaults=defaults
         )
         cdm.clean_fields()
         return (cdm, created,)
 
-    def load(self, date_for=None, force_update=False, **_kwargs):
+    def load(self, date_for=None, ed_next=False, force_update=False, **_kwargs):
         """
         TODO: clean up how we do this. We want to be able to call the loader
         with an existing data set (not having to call the extractor) but we
@@ -334,13 +400,9 @@ class CourseDailyMetricsLoader(object):
         Raises ValidationError if invalid data is attempted to be saved to the
         course daily metrics model instance
         """
-        if not date_for:
-            date_for = prev_day(
-                datetime.datetime.utcnow().replace(tzinfo=utc).date())
-        else:
-            date_for = as_datetime(date_for).replace(tzinfo=utc)
+        date_for = pipeline_date_for_rule(date_for)
         try:
-            cdm = CourseDailyMetrics.objects.get(course_id=self.course_id,
+            cdm = CourseDailyMetrics.objects.get(course_id=str(self.course_id),
                                                  date_for=date_for)
             # record found, only update if force update flag is True
             if not force_update:
@@ -349,5 +411,5 @@ class CourseDailyMetricsLoader(object):
             # record not found, move on to creating
             pass
 
-        data = self.get_data(date_for=date_for)
+        data = self.get_data(date_for=date_for, ed_next=ed_next)
         return self.save_metrics(date_for=date_for, data=data)
