@@ -1,10 +1,12 @@
 """Figures views
 """
 
+from __future__ import absolute_import
 from datetime import datetime
+import logging
+from django.conf import settings
 from django.contrib.auth import get_user_model
 from django.contrib.auth.decorators import login_required, user_passes_test
-import django.contrib.sites.shortcuts
 from django.contrib.sites.models import Site
 from django.http import HttpResponseRedirect
 from django.shortcuts import get_object_or_404, render
@@ -12,29 +14,30 @@ from django.views.decorators.csrf import ensure_csrf_cookie
 
 from rest_framework import viewsets
 from rest_framework.authentication import (
-    BasicAuthentication,
     SessionAuthentication,
     TokenAuthentication,
 )
 from rest_framework.decorators import detail_route, list_route
 from rest_framework.exceptions import NotFound
 from rest_framework.permissions import IsAuthenticated
+
 from rest_framework.filters import (
-    DjangoFilterBackend,
     SearchFilter,
-    OrderingFilter
+    OrderingFilter  # TODO Is this backward compatible? fixes test_course_data_view
 )
 from rest_framework.response import Response
 from rest_framework.views import APIView
 
+# https://www.django-rest-framework.org/api-guide/filtering/#searchfilter
+try:
+    from django_filters.rest_framework import DjangoFilterBackend
+except ImportError:
+    from rest_framework.filters import DjangoFilterBackend  # pylint: disable=ungrouped-imports
+
 from opaque_keys import InvalidKeyError
 from opaque_keys.edx.keys import CourseKey
 
-# Directly including edx-platform objects for early development
-# Follow-on, we'll likely consolidate edx-platform model imports to an adapter
-from openedx.core.djangoapps.content.course_overviews.models import CourseOverview  # noqa pylint: disable=import-error
-from student.models import CourseEnrollment  # pylint: disable=import-error
-
+from figures.compat import CourseEnrollment, CourseOverview
 from figures.filters import (
     CourseDailyMetricsFilter,
     CourseEnrollmentFilter,
@@ -53,6 +56,7 @@ from figures.models import (
     SiteDailyMetrics,
     SiteMauMetrics,
 )
+from figures.query import site_users_enrollment_data
 from figures.serializers import (
     CourseCompletedSerializer,
     CourseDailyMetricsSerializer,
@@ -61,10 +65,12 @@ from figures.serializers import (
     CourseIndexSerializer,
     CourseMauMetricsSerializer,
     CourseMauLiveMetricsSerializer,
+    CourseOverviewSerializer,
     EnrollmentMetricsSerializer,
     GeneralCourseDataSerializer,
     LearnerDetailsSerializer,
     LearnerMetricsSerializer,
+    LearnerMetricsSerializerV2,
     SiteDailyMetricsSerializer,
     SiteMauMetricsSerializer,
     SiteMauLiveMetricsSerializer,
@@ -89,11 +95,13 @@ from figures.mau import (
 
 UNAUTHORIZED_USER_REDIRECT_URL = '/'
 
+logger = logging.getLogger(__name__)
 
 #
 # UI Template rendering views
 #
 
+
 @ensure_csrf_cookie
 @login_required
 @user_passes_test(lambda u: u.is_active,
@@ -127,7 +135,6 @@ class CommonAuthMixin(object):
     TODO: Consider moving this to figures.permissions
     '''
     authentication_classes = (
-        BasicAuthentication,
         SessionAuthentication,
         TokenAuthentication,
     )
@@ -142,7 +149,6 @@ class StaffUserOnDefaultSiteAuthMixin(object):
     TODO: Consider moving this to figures.permissions
     '''
     authentication_classes = (
-        BasicAuthentication,
         SessionAuthentication,
         TokenAuthentication,
     )
@@ -151,40 +157,78 @@ class StaffUserOnDefaultSiteAuthMixin(object):
         figures.permissions.IsStaffUserOnDefaultSite,
     )
 
-#
-# Views for data in edX platform
-#
 
-
-# @view_auth_classes(is_authenticated=True)
-class CoursesIndexViewSet(CommonAuthMixin, viewsets.ReadOnlyModelViewSet):
-    '''Provides a list of courses with abbreviated details
-
-    Uses figures.filters.CourseOverviewFilter to select subsets of
-    CourseOverview objects
-
-    We want to be able to filter on
-    - org: exact and search
-    - name: exact and search
-    - description search
-    - enrollment start
-    - enrollment end
-    - start
-    - end
-
-    '''
+class CourseOverviewViewSet(CommonAuthMixin, viewsets.ReadOnlyModelViewSet):
+    """Base class for ViewSet classes that are based on CourseOverview
+    """
     model = CourseOverview
+    serializer_class = CourseOverviewSerializer
     pagination_class = FiguresLimitOffsetPagination
-    serializer_class = CourseIndexSerializer
-
     filter_backends = (DjangoFilterBackend, )
     filter_class = CourseOverviewFilter
+    lookup_value_regex = settings.COURSE_ID_PATTERN
 
     def get_queryset(self):
-        site = django.contrib.sites.shortcuts.get_current_site(self.request)
+        site = figures.sites.get_requested_site(self.request)
         queryset = figures.sites.get_courses_for_site(site)
         return queryset
 
+    def get_course_key(self, course_id_str):
+        """
+        Initial quick refactoring. Next, make this a decorator we can use for
+        the methods that need to derive a course_key from a course id string
+
+        TODO: Look at the `site_course_helper` method further down in
+        `CourseMonthlyMetricsViewSet`. It checks if a course is in a site. If
+        not, then raises NotFound.
+        """
+        try:
+            return CourseKey.from_string(course_id_str.replace(' ', '+'))
+        except InvalidKeyError:
+            label = self.__class__.__name__
+            logger.error('figures {}: InvalidKeyError: {}'.format(label,
+                                                                  course_id_str))
+            raise NotFound()
+
+    def retrieve(self, request, *args, **kwargs):
+        course_key = self.get_course_key(
+            kwargs.get('pk', ''))
+        site = figures.sites.get_requested_site(self.request)
+        if figures.helpers.is_multisite():
+            if site != figures.sites.get_site_for_course(course_key):
+                # Raising NotFound instead of PermissionDenied
+                raise NotFound()
+        course_overview = get_object_or_404(CourseOverview, pk=course_key)
+        return Response(self.get_serializer_class()(course_overview).data)
+
+
+class CoursesIndexViewSet(CourseOverviewViewSet):
+    """Provides a list of courses with abbreviated details
+    """
+    serializer_class = CourseIndexSerializer
+
+
+class GeneralCourseDataViewSet(CourseOverviewViewSet):
+    """General course data
+    """
+    serializer_class = GeneralCourseDataSerializer
+    # The "kilo paginator"  is a tempoarary hack to return all course to not
+    # have to change the front end until Figures "Level 2"
+    pagination_class = FiguresKiloPagination
+    filter_backends = (SearchFilter, DjangoFilterBackend, OrderingFilter)
+    search_fields = ['display_name', 'id']
+    ordering_fields = ['display_name', 'self_paced', 'date_joined']
+
+
+class CourseDetailsViewSet(CommonAuthMixin, viewsets.ReadOnlyModelViewSet):
+    """Detailed course data
+    """
+    serializer_class = CourseDetailsSerializer
+    # The "kilo paginator"  is a tempoarary hack to return all course to not
+    # have to change the front end until Figures "Level 2"
+    pagination_class = FiguresKiloPagination
+    filter_backends = (DjangoFilterBackend, )
+
 
 class UserIndexViewSet(CommonAuthMixin, viewsets.ReadOnlyModelViewSet):
     '''Provides a list of users with abbreviated details
@@ -198,7 +242,7 @@ class UserIndexViewSet(CommonAuthMixin, viewsets.ReadOnlyModelViewSet):
     filter_class = UserFilterSet
 
     def get_queryset(self):
-        site = django.contrib.sites.shortcuts.get_current_site(self.request)
+        site = figures.sites.get_requested_site(self.request)
         queryset = figures.sites.get_users_for_site(site)
         return queryset
 
@@ -211,26 +255,21 @@ class CourseEnrollmentViewSet(CommonAuthMixin, viewsets.ReadOnlyModelViewSet):
     filter_class = CourseEnrollmentFilter
 
     def get_queryset(self):
-        site = django.contrib.sites.shortcuts.get_current_site(self.request)
+        site = figures.sites.get_requested_site(self.request)
         queryset = figures.sites.get_course_enrollments_for_site(site)
         return queryset
 
-#
-# Views for Figures models
-#
-
 
 class CourseDailyMetricsViewSet(CommonAuthMixin, viewsets.ModelViewSet):
 
     model = CourseDailyMetrics
-    # queryset = CourseDailyMetrics.objects.all()
     pagination_class = FiguresLimitOffsetPagination
     serializer_class = CourseDailyMetricsSerializer
     filter_backends = (DjangoFilterBackend, )
     filter_class = CourseDailyMetricsFilter
 
     def get_queryset(self):
-        site = django.contrib.sites.shortcuts.get_current_site(self.request)
+        site = figures.sites.get_requested_site(self.request)
         queryset = CourseDailyMetrics.objects.filter(site=site)
         return queryset
 
@@ -244,16 +283,11 @@ class SiteDailyMetricsViewSet(CommonAuthMixin, viewsets.ModelViewSet):
     filter_class = SiteDailyMetricsFilter
 
     def get_queryset(self):
-        site = django.contrib.sites.shortcuts.get_current_site(self.request)
+        site = figures.sites.get_requested_site(self.request)
         queryset = SiteDailyMetrics.objects.filter(site=site)
         return queryset
 
 
-#
-# Views for the front end
-#
-
-
 class GeneralSiteMetricsView(CommonAuthMixin, APIView):
     """Viewset intended for Figures Web UI
 
@@ -281,7 +315,7 @@ class GeneralSiteMetricsView(CommonAuthMixin, APIView):
         '''
         Does not yet support multi-tenancy
         '''
-        site = django.contrib.sites.shortcuts.get_current_site(request)
+        site = figures.sites.get_requested_site(request)
         date_for = request.query_params.get('date_for')
         data = self.metrics_method(site=site, date_for=date_for)
 
@@ -292,69 +326,6 @@ class GeneralSiteMetricsView(CommonAuthMixin, APIView):
         return Response(data)
 
 
-class GeneralCourseDataViewSet(CommonAuthMixin, viewsets.ReadOnlyModelViewSet):
-    """Viewset intended for Figures Web UI
-    """
-    model = CourseOverview
-
-    # The "kilo paginator"  is a tempoarary hack to return all course to not
-    # have to change the front end until Figures "Level 2"
-    pagination_class = FiguresKiloPagination
-    serializer_class = GeneralCourseDataSerializer
-    filter_backends = (SearchFilter, DjangoFilterBackend, OrderingFilter)
-    filter_class = CourseOverviewFilter
-    search_fields = ['display_name', 'id']
-    ordering_fields = ['display_name', 'self_paced', 'date_joined']
-
-    def get_queryset(self):
-        site = django.contrib.sites.shortcuts.get_current_site(self.request)
-        queryset = figures.sites.get_courses_for_site(site)
-        return queryset
-
-    def retrieve(self, request, *args, **kwargs):
-        course_id_str = kwargs.get('pk', '')
-        course_key = CourseKey.from_string(course_id_str.replace(' ', '+'))
-        site = django.contrib.sites.shortcuts.get_current_site(request)
-        if figures.helpers.is_multisite():
-            if site != figures.sites.get_site_for_course(course_key):
-                # Raising NotFound instead of PermissionDenied
-                raise NotFound()
-        course_overview = get_object_or_404(CourseOverview, pk=course_key)
-        return Response(GeneralCourseDataSerializer(course_overview).data)
-
-
-class CourseDetailsViewSet(CommonAuthMixin, viewsets.ReadOnlyModelViewSet):
-    '''
-
-    '''
-    model = CourseOverview
-
-    # The "kilo paginator"  is a tempoarary hack to return all course to not
-    # have to change the front end until Figures "Level 2"
-    pagination_class = FiguresKiloPagination
-    serializer_class = CourseDetailsSerializer
-    filter_backends = (DjangoFilterBackend, )
-    filter_class = CourseOverviewFilter
-
-    def get_queryset(self):
-        site = django.contrib.sites.shortcuts.get_current_site(self.request)
-        queryset = figures.sites.get_courses_for_site(site)
-        return queryset
-
-    def retrieve(self, request, *args, **kwargs):
-        # NOTE: Duplicating code in GeneralCourseDataViewSet. Candidate to dry up
-        # Make it a decorator
-        course_id_str = kwargs.get('pk', '')
-        course_key = CourseKey.from_string(course_id_str.replace(' ', '+'))
-        site = django.contrib.sites.shortcuts.get_current_site(request)
-        if figures.helpers.is_multisite():
-            if site != figures.sites.get_site_for_course(course_key):
-                # Raising NotFound instead of PermissionDenied
-                raise NotFound()
-        course_overview = get_object_or_404(CourseOverview, pk=course_key)
-        return Response(CourseDetailsSerializer(course_overview).data)
-
-
 class GeneralUserDataViewSet(CommonAuthMixin, viewsets.ReadOnlyModelViewSet):
     '''View class to serve general user data to the Figures UI
 
@@ -376,7 +347,7 @@ class GeneralUserDataViewSet(CommonAuthMixin, viewsets.ReadOnlyModelViewSet):
     ordering_fields = ['username', 'email', 'profile__name', 'is_active', 'date_joined']
 
     def get_queryset(self):
-        site = django.contrib.sites.shortcuts.get_current_site(self.request)
+        site = figures.sites.get_requested_site(self.request)
         queryset = figures.sites.get_users_for_site(site)
         return queryset
 
@@ -389,20 +360,21 @@ class LearnerDetailsViewSet(CommonAuthMixin, viewsets.ReadOnlyModelViewSet):
     filter_class = UserFilterSet
 
     def get_queryset(self):
-        site = django.contrib.sites.shortcuts.get_current_site(self.request)
+        site = figures.sites.get_requested_site(self.request)
         queryset = figures.sites.get_users_for_site(site)
         return queryset
 
     def get_serializer_context(self):
         context = super(LearnerDetailsViewSet, self).get_serializer_context()
-        context['site'] = django.contrib.sites.shortcuts.get_current_site(self.request)
+        context['site'] = figures.sites.get_requested_site(self.request)
         return context
 
 
-class LearnerMetricsViewSet(CommonAuthMixin, viewsets.ReadOnlyModelViewSet):
+class LearnerMetricsViewSetV1(CommonAuthMixin, viewsets.ReadOnlyModelViewSet):
     """Provides user identity and nested enrollment data
 
-    This view is unders active development and subject to change
+    Renamed class by appending 'V1' as we are retaining it only to compare for
+    performance testing and verify identical results from the new viewset
 
     TODO: After we get this class tests running, restructure this module:
     * Group all user model based viewsets together
@@ -420,14 +392,114 @@ class LearnerMetricsViewSet(CommonAuthMixin, viewsets.ReadOnlyModelViewSet):
     search_fields = ['username', 'email', 'profile__name']
     ordering_fields = ['username', 'email', 'profile__name', 'is_active', 'date_joined']
 
+    def query_param_course_keys(self):
+        """
+        TODO: Mixin this
+        """
+        cid_list = self.request.GET.getlist('course')
+        return [CourseKey.from_string(elem.replace(' ', '+')) for elem in cid_list]
+
+    def get_enrolled_users(self, site, course_keys):
+        """Get users enrolled in the specific courses for the specified site
+
+        Args:
+            site: The site for which is being called
+            course_keys: list of Open edX course keys
+
+        Returns:
+            Django QuerySet of users enrolled in the specified courses
+
+        Note:
+            We should move this to `figures.sites`
+        """
+        qs = figures.sites.get_users_for_site(site).filter(
+            courseenrollment__course_id__in=course_keys
+            ).select_related('profile').prefetch_related('courseenrollment_set')
+        return qs.distinct()
+
     def get_queryset(self):
-        site = django.contrib.sites.shortcuts.get_current_site(self.request)
-        queryset = figures.sites.get_users_for_site(site)
-        return queryset
+        """
+        If one or more course keys are given as query parameters, then
+        * Course key filtering mode is ued. Any invalid keys are filtered out
+          from the list
+        * If no valid course keys are found, then an empty list is returned from
+          this view
+        """
+        site = figures.sites.get_requested_site(self.request)
+        course_keys = figures.sites.get_course_keys_for_site(site)
+        try:
+            param_course_keys = self.query_param_course_keys()
+        except InvalidKeyError:
+            raise NotFound()
+        if param_course_keys:
+            if not set(param_course_keys).issubset(set(course_keys)):
+                raise NotFound()
+            else:
+                course_keys = param_course_keys
+        return self.get_enrolled_users(site=site, course_keys=course_keys)
 
     def get_serializer_context(self):
-        context = super(LearnerMetricsViewSet, self).get_serializer_context()
-        context['site'] = django.contrib.sites.shortcuts.get_current_site(self.request)
+        context = super(LearnerMetricsViewSetV1, self).get_serializer_context()
+        context['site'] = figures.sites.get_requested_site(self.request)
+        context['course_keys'] = self.query_param_course_keys()
+        return context
+
+
+class LearnerMetricsViewSetV2(CommonAuthMixin, viewsets.ReadOnlyModelViewSet):
+    """Provides user identity and nested enrollment data
+
+    Version 2 of this viewset. We'll remove the old view
+    This view is unders active development and subject to change
+
+    TODO: After we get this class tests running, restructure this module:
+    * Group all user model based viewsets together
+    * Make a base user viewset with the `get_queryset` and `get_serializer_context`
+      methods
+    """
+    model = get_user_model()
+    pagination_class = FiguresLimitOffsetPagination
+    serializer_class = LearnerMetricsSerializerV2
+    filter_backends = (SearchFilter, DjangoFilterBackend, OrderingFilter)
+    filter_class = UserFilterSet
+
+    search_fields = ['username', 'email', 'profile__name']
+    ordering_fields = [
+        'username', 'email', 'profile__name', 'is_active', 'date_joined'
+    ]
+
+    def query_param_course_ids(self):
+        """Returns list of formatted course ids or empty list
+
+        Important: This method does not validate the course id format or if the
+        course id exists in the site
+
+        Each course id is in its own 'course' query param. We can have multiple
+        'course' query params
+
+        Example query params:
+            `endpoint/?course=apple&course=banana&course=cherry`
+
+        If no 'course' parameters then an empty list is returned
+        """
+        course_id_list = self.request.GET.getlist('course')
+        return [elem.replace(' ', '+') for elem in course_id_list]
+
+    def get_queryset(self):
+        """
+        If one or more course keys are given as query parameters, then
+        * Course key filtering mode is ued. Any invalid keys are filtered out
+          from the list
+        * If no valid course keys are found, then an empty list is returned from
+          this view
+        """
+        site = figures.sites.get_requested_site(self.request)
+        course_ids = self.query_param_course_ids()
+        return site_users_enrollment_data(site=site, course_ids=course_ids)
+
+    def get_serializer_context(self):
+        context = super(LearnerMetricsViewSetV2, self).get_serializer_context()
+        context['site'] = figures.sites.get_requested_site(self.request)
+        context['course_ids'] = self.query_param_course_ids()
         return context
 
 
@@ -448,7 +520,7 @@ class EnrollmentMetricsViewSet(CommonAuthMixin, viewsets.ReadOnlyModelViewSet):
     filter_class = EnrollmentMetricsFilter
 
     def get_queryset(self):
-        site = django.contrib.sites.shortcuts.get_current_site(self.request)
+        site = figures.sites.get_requested_site(self.request)
         queryset = LearnerCourseGradeMetrics.objects.filter(site=site)
         return queryset
 
@@ -461,7 +533,7 @@ class EnrollmentMetricsViewSet(CommonAuthMixin, viewsets.ReadOnlyModelViewSet):
         The default router does not support hyphen in the custom action, so
         we need to use the underscore until we implement a custom router
         """
-        site = django.contrib.sites.shortcuts.get_current_site(request)
+        site = figures.sites.get_requested_site(request)
         qs = self.model.objects.completed_ids_for_site(site=site)
         page = self.paginate_queryset(qs)
         if page is not None:
@@ -479,7 +551,7 @@ class EnrollmentMetricsViewSet(CommonAuthMixin, viewsets.ReadOnlyModelViewSet):
         Return matching LearnerCourseGradeMetric rows that have completed
         enrollments
         """
-        site = django.contrib.sites.shortcuts.get_current_site(request)
+        site = figures.sites.get_requested_site(request)
         qs = self.model.objects.completed_for_site(site=site)
         page = self.paginate_queryset(qs)
         if page is not None:
@@ -490,10 +562,8 @@ class EnrollmentMetricsViewSet(CommonAuthMixin, viewsets.ReadOnlyModelViewSet):
 
 
 class CourseMonthlyMetricsViewSet(CommonAuthMixin, viewsets.ViewSet):
-    """
-
-    """
 
+    lookup_value_regex = settings.COURSE_ID_PATTERN
     # TODO: Make 'months_back' be a query parameter.
     # We will also need to either set a limit or paginate history results
     months_back = 6
@@ -511,7 +581,7 @@ class CourseMonthlyMetricsViewSet(CommonAuthMixin, viewsets.ViewSet):
         except InvalidKeyError:
             raise NotFound()
 
-        site = django.contrib.sites.shortcuts.get_current_site(self.request)
+        site = figures.sites.get_requested_site(self.request)
         if figures.helpers.is_multisite():
             if site != figures.sites.get_site_for_course(course_id):
                 raise NotFound()
@@ -538,7 +608,7 @@ class CourseMonthlyMetricsViewSet(CommonAuthMixin, viewsets.ViewSet):
         TODO: NEXT Add query params to get data from previous months
         TODO: Add paginagation
         """
-        site = django.contrib.sites.shortcuts.get_current_site(request)
+        site = figures.sites.get_requested_site(request)
         course_keys = figures.sites.get_course_keys_for_site(site)
         date_for = datetime.utcnow().date()
         month_for = '{}/{}'.format(date_for.month, date_for.year)
@@ -634,18 +704,19 @@ class SiteMonthlyMetricsViewSet(CommonAuthMixin, viewsets.ViewSet):
     Tradeoff: Additional storage cost to reduced request time
     Perhaps we make this a server setting
     """
+
     def list(self, request):
         """
         Returns site metrics data for current month
         """
 
-        site = django.contrib.sites.shortcuts.get_current_site(request)
+        site = figures.sites.get_requested_site(request)
         data = metrics.get_current_month_site_metrics(site)
         return Response(data)
 
     @list_route()
     def registered_users(self, request):
-        site = django.contrib.sites.shortcuts.get_current_site(request)
+        site = figures.sites.get_requested_site(request)
         date_for = datetime.utcnow().date()
         months_back = 6
 
@@ -663,7 +734,7 @@ class SiteMonthlyMetricsViewSet(CommonAuthMixin, viewsets.ViewSet):
         """
         TODO: Rename the metrics module function to "new_users" to match this
         """
-        site = django.contrib.sites.shortcuts.get_current_site(request)
+        site = figures.sites.get_requested_site(request)
         date_for = datetime.utcnow().date()
         months_back = 6
 
@@ -678,7 +749,7 @@ class SiteMonthlyMetricsViewSet(CommonAuthMixin, viewsets.ViewSet):
 
     @list_route()
     def course_completions(self, request):
-        site = django.contrib.sites.shortcuts.get_current_site(request)
+        site = figures.sites.get_requested_site(request)
         date_for = datetime.utcnow().date()
         months_back = 6
 
@@ -693,7 +764,7 @@ class SiteMonthlyMetricsViewSet(CommonAuthMixin, viewsets.ViewSet):
 
     @list_route()
     def course_enrollments(self, request):
-        site = django.contrib.sites.shortcuts.get_current_site(request)
+        site = figures.sites.get_requested_site(request)
         date_for = datetime.utcnow().date()
         months_back = 6
 
@@ -708,7 +779,7 @@ class SiteMonthlyMetricsViewSet(CommonAuthMixin, viewsets.ViewSet):
 
     @list_route()
     def site_courses(self, request):
-        site = django.contrib.sites.shortcuts.get_current_site(request)
+        site = figures.sites.get_requested_site(request)
         date_for = datetime.utcnow().date()
         months_back = 6
 
@@ -723,7 +794,7 @@ class SiteMonthlyMetricsViewSet(CommonAuthMixin, viewsets.ViewSet):
 
     @list_route()
     def active_users(self, request):
-        site = django.contrib.sites.shortcuts.get_current_site(request)
+        site = figures.sites.get_requested_site(request)
         months_back = 6
         active_users = metrics.get_site_mau_history_metrics(site=site,
                                                             months_back=months_back)
@@ -732,6 +803,7 @@ class SiteMonthlyMetricsViewSet(CommonAuthMixin, viewsets.ViewSet):
 
 class CourseMauLiveMetricsViewSet(CommonAuthMixin, viewsets.GenericViewSet):
     serializer_class = CourseMauLiveMetricsSerializer
+    lookup_value_regex = settings.COURSE_ID_PATTERN
 
     def get_queryset(self):
         """
@@ -743,7 +815,7 @@ class CourseMauLiveMetricsViewSet(CommonAuthMixin, viewsets.GenericViewSet):
     def retrieve(self, request, **kwargs):
         course_id_str = kwargs.get('pk', '')
         course_key = CourseKey.from_string(course_id_str.replace(' ', '+'))
-        site = django.contrib.sites.shortcuts.get_current_site(request)
+        site = figures.sites.get_requested_site(request)
 
         if figures.helpers.is_multisite():
             if site != figures.sites.get_site_for_course(course_key):
@@ -754,7 +826,7 @@ class CourseMauLiveMetricsViewSet(CommonAuthMixin, viewsets.GenericViewSet):
         return Response(serializer.data)
 
     def list(self, request):
-        site = django.contrib.sites.shortcuts.get_current_site(request)
+        site = figures.sites.get_requested_site(request)
         course_overviews = figures.sites.get_courses_for_site(site)
         data = []
         for co in course_overviews:
@@ -785,7 +857,7 @@ class SiteMauLiveMetricsViewSet(CommonAuthMixin, viewsets.GenericViewSet):
         We use list instead of retrieve because retrieve requires a resource
         identifier, like a PK
         """
-        site = django.contrib.sites.shortcuts.get_current_site(request)
+        site = figures.sites.get_requested_site(request)
         data = retrieve_live_site_mau_data(site)
         serializer = self.serializer_class(data)
         return Response(serializer.data)
@@ -796,9 +868,10 @@ class CourseMauMetricsViewSet(CommonAuthMixin, viewsets.ReadOnlyModelViewSet):
     serializer_class = CourseMauMetricsSerializer
     filter_backends = (DjangoFilterBackend, )
     filter_class = CourseMauMetricsFilter
+    lookup_value_regex = settings.COURSE_ID_PATTERN
 
     def get_queryset(self):
-        site = django.contrib.sites.shortcuts.get_current_site(self.request)
+        site = figures.sites.get_requested_site(self.request)
         queryset = CourseMauMetrics.objects.filter(site=site)
         return queryset
 
@@ -811,7 +884,7 @@ class SiteMauMetricsViewSet(CommonAuthMixin, viewsets.ReadOnlyModelViewSet):
     filter_class = SiteMauMetricsFilter
 
     def get_queryset(self):
-        site = django.contrib.sites.shortcuts.get_current_site(self.request)
+        site = figures.sites.get_requested_site(self.request)
         queryset = SiteMauMetrics.objects.filter(site=site)
         return queryset
 
@@ -822,8 +895,10 @@ class SiteViewSet(StaffUserOnDefaultSiteAuthMixin, viewsets.ReadOnlyModelViewSet
     Access is restricted to global (Django instance) staff
     """
     model = Site
-    queryset = Site.objects.all()
     pagination_class = FiguresLimitOffsetPagination
     serializer_class = SiteSerializer
     filter_backends = (DjangoFilterBackend, )
     filter_class = SiteFilterSet
+
+    def get_queryset(self):
+        return figures.sites.get_sites()
