@@ -4,10 +4,10 @@ const apiConfig = {
   edxUserInfoApi: '/api/user/v1/accounts/',
   courseEnrollmentsApi: '/figures/api/course-enrollments/',
   generalSiteMetrics: '/figures/api/general-site-metrics/',
-  coursesGeneral: '/figures/api/courses/general/',
-  coursesDetailed: '/figures/api/courses/detail/',
-  learnersGeneral: '/figures/api/users/general/',
-  learnersDetailed: '/figures/api/users/detail/',
+  coursesGeneral: '/figures/api/courses-general/',
+  coursesDetailed: '/figures/api/courses-detail/',
+  learnersGeneral: '/figures/api/users-general/',
+  learnersDetailed: '/figures/api/users-detail/',
   reportingCsvReportsApi: '/reporting/api/csv-reports/',
   coursesIndex: '/figures/api/courses-index/',
   learnerMetrics: '/figures/api/learner-metrics/',
