@@ -7,6 +7,7 @@ Uses Factory Boy: https://factoryboy.readthedocs.io/en/latest/
 
 '''
 
+from __future__ import absolute_import
 import datetime
 from dateutil.relativedelta import relativedelta
 from django.utils.timezone import utc
@@ -25,18 +26,19 @@ from openedx.core.djangoapps.course_groups.models import (
     CourseUserGroup,
     CohortMembership,
 )
-from openedx.core.djangoapps.xmodule_django.models import CourseKeyField
-from courseware.models import StudentModule
+
+from figures.compat import StudentModule, CourseKeyField, GeneratedCertificate
+
 from student.models import CourseAccessRole, CourseEnrollment, UserProfile
 from lms.djangoapps.teams.models import CourseTeam, CourseTeamMembership
 
 import organizations
 
-from figures.compat import GeneratedCertificate
 from figures.helpers import as_course_key
 from figures.models import (
     CourseDailyMetrics,
     CourseMauMetrics,
+    EnrollmentData,
     LearnerCourseGradeMetrics,
     SiteDailyMetrics,
     SiteMonthlyMetrics,
@@ -48,6 +50,7 @@ from tests.helpers import (
     OPENEDX_RELEASE,
     GINKGO,
 )
+import six
 
 
 COURSE_ID_STR_TEMPLATE = 'course-v1:StarFleetAcademy+SFA{}+2161'
@@ -73,7 +76,7 @@ class UserProfileFactory(DjangoModelFactory):
         ['p','m','b','a','hs','jh','el','none', 'other',]
         )
     profile_image_uploaded_at = fuzzy.FuzzyDateTime(datetime.datetime(
-        2018,04,01, tzinfo=factory.compat.UTC))
+        2018,0o4,0o1, tzinfo=factory.compat.UTC))
 
 
 class UserFactory(DjangoModelFactory):
@@ -87,7 +90,7 @@ class UserFactory(DjangoModelFactory):
     is_staff = False
     is_superuser = False
     date_joined = fuzzy.FuzzyDateTime(datetime.datetime(
-        2018,04,01, tzinfo=factory.compat.UTC))
+        2018, 4, 1, tzinfo=factory.compat.UTC))
 
     # TODO: Figure out if this can be a SubFactory and the advantages
     profile = factory.RelatedFactory(UserProfileFactory, 'user')
@@ -213,9 +216,23 @@ class StudentModuleFactory(DjangoModelFactory):
     course_id = factory.Sequence(lambda n: as_course_key(
         COURSE_ID_STR_TEMPLATE.format(n)))
     created = fuzzy.FuzzyDateTime(datetime.datetime(
-        2018,02,02, tzinfo=factory.compat.UTC))
+        2018,2,2, tzinfo=factory.compat.UTC))
     modified = fuzzy.FuzzyDateTime(datetime.datetime(
-        2018,02,02, tzinfo=factory.compat.UTC))
+        2018,2,2, tzinfo=factory.compat.UTC))
+
+
+    @classmethod
+    def from_course_enrollment(cls, course_enrollment, **kwargs):
+        """Contruct a StudentModule  for the given CourseEnrollment
+
+        kwargs provides for additional optional parameters if you need to
+        override the default factory assignment
+        """
+        kwargs.update({
+            'student': course_enrollment.user,
+            'course_id': course_enrollment.course_id,
+            })
+        return cls(**kwargs)
 
 
 if OPENEDX_RELEASE == GINKGO:
@@ -255,7 +272,7 @@ else:
                 course_id = kwargs.get('course_id')
                 course_overview = None
                 if course_id is not None:
-                    if isinstance(course_id, basestring):
+                    if isinstance(course_id, six.string_types):
                         course_id = as_course_key(course_id)
                         course_kwargs.setdefault('id', course_id)
 
@@ -301,6 +318,29 @@ class CourseDailyMetricsFactory(DjangoModelFactory):
     num_learners_completed = 5
 
 
+class EnrollmentDataFactory(DjangoModelFactory):
+    class Meta:
+        model = EnrollmentData
+
+    site = factory.SubFactory(SiteFactory)
+    user = factory.SubFactory(UserFactory)
+    course_id = factory.Sequence(lambda n:
+        'course-v1:StarFleetAcademy+SFA{}+2161'.format(n))
+    date_for = factory.Sequence(lambda n:
+        (datetime.datetime(2018, 1, 1) + datetime.timedelta(
+            days=n)).replace(tzinfo=utc).date())
+    date_enrolled = factory.Sequence(lambda n:
+        (datetime.datetime(2018, 1, 1) + datetime.timedelta(
+            days=n)).replace(tzinfo=utc).date())
+    is_enrolled = True
+    is_completed = False
+    progress_percent = 0.50
+    points_possible = 30.0
+    points_earned = 15.0
+    sections_worked = 5
+    sections_possible = 10
+
+
 class LearnerCourseGradeMetricsFactory(DjangoModelFactory):
     class Meta:
         model = LearnerCourseGradeMetrics
