@@ -3,16 +3,17 @@
 TODO: Update this test module to test multisite environments
 """
 
+from __future__ import absolute_import
 import datetime
 import mock
 import pytest
 
+from dateutil.relativedelta import relativedelta
+
 from django.core.exceptions import PermissionDenied, ValidationError
 
-from student.models import CourseEnrollment, CourseAccessRole
-
+from figures.compat import CourseAccessRole, CourseEnrollment
 from figures.helpers import as_datetime, next_day, prev_day
-
 from figures.models import CourseDailyMetrics, PipelineError
 from figures.pipeline import course_daily_metrics as pipeline_cdm
 import figures.sites
@@ -33,6 +34,7 @@ from tests.helpers import (
     OPENEDX_RELEASE,
     GINKGO,
 )
+from six.moves import range
 
 
 if organizations_support_sites():
@@ -235,16 +237,62 @@ class TestCourseDailyMetricsExtractor(object):
     def setup(self, db):
         self.course_enrollments = [CourseEnrollmentFactory() for i in range(1, 5)]
         self.student_module = StudentModuleFactory()
+        self.date_for = datetime.datetime.utcnow().date()
 
-    def test_extract(self, monkeypatch):
+    def test_extract_default(self, monkeypatch):
+        """Default progress calculator is called when `ed_next` param not set
+        """
+        expected_avg_prog = 'fake-average-progress'
         course_id = self.course_enrollments[0].course_id
         monkeypatch.setattr(figures.pipeline.course_daily_metrics,
                             'bulk_calculate_course_progress_data',
-                            lambda **_kwargs: dict(average_progress=0.5))
+                            lambda **_kwargs: dict(average_progress=expected_avg_prog))
 
-        results = pipeline_cdm.CourseDailyMetricsExtractor().extract(course_id)
+        results = pipeline_cdm.CourseDailyMetricsExtractor().extract(
+            course_id, self.date_for)
+        assert results['average_progress'] == expected_avg_prog
+
+    @pytest.mark.parametrize('prog_func, ed_next', [
+            ('bulk_calculate_course_progress_data', False),
+            ('calculate_course_progress_next', True)
+        ])
+    def test_extract_ed_next(self, monkeypatch, prog_func, ed_next):
+        """Tests default and alternate progress calculators
+        """
+        course_id = self.course_enrollments[0].course_id
+        prog_func_str = 'figures.pipeline.course_daily_metrics.{}'.format(prog_func)
+        with mock.patch(prog_func_str) as prog_mock:
+            results = pipeline_cdm.CourseDailyMetricsExtractor().extract(
+                course_id, self.date_for, ed_next=ed_next)
+            assert prog_mock.called
         assert results
 
+    @pytest.mark.parametrize('prog_func, ed_next', [
+            ('bulk_calculate_course_progress_data', False),
+            ('calculate_course_progress_next', True)
+        ])
+    def test_when_calculate_course_progress_data_fails(self,
+                                                       monkeypatch,
+                                                       caplog,
+                                                       prog_func,
+                                                       ed_next):
+        course_id = self.course_enrollments[0].course_id
+
+        def prog_func_mock(**_kwargs):
+            raise Exception('fake exception')
+
+        monkeypatch.setattr(figures.pipeline.course_daily_metrics,
+                            prog_func,
+                            prog_func_mock)
+
+        results = pipeline_cdm.CourseDailyMetricsExtractor().extract(
+            course_id, self.date_for, ed_next)
+
+        last_log = caplog.records[-1]
+        assert last_log.message.startswith(
+            'FIGURES:FAIL {}'.format(prog_func))
+        assert not results['average_progress']
+
 
 @pytest.mark.django_db
 class TestCourseDailyMetricsLoader(object):
@@ -272,7 +320,7 @@ class TestCourseDailyMetricsLoader(object):
         else:
             course_id = self.course_enrollments[0].course.id
 
-        def get_data(self, date_for):
+        def get_data(self, date_for, ed_next=False):
             return {
                 'average_progress': 1.0,
                 'num_learners_completed': 2,
@@ -313,6 +361,16 @@ class TestCourseDailyMetricsLoader(object):
             assert CourseDailyMetrics.objects.count() == 0
             assert 'average_progress' in execinfo.value.message_dict
 
+    @pytest.mark.parametrize(['days_back', 'expected_val'], [(1, 1), (2, None)])
+    def test_average_progress_yesterday_or_prior(self, monkeypatch, days_back, expected_val):
+        monkeypatch.setattr(figures.pipeline.course_daily_metrics,
+                            'bulk_calculate_course_progress_data',
+                            lambda **_kwargs: dict(average_progress=1))
+        course_id = self.course_enrollments[0].course_id
+        date_for = datetime.datetime.today().date() - relativedelta(days=days_back)
+        cdm, created = pipeline_cdm.CourseDailyMetricsLoader(course_id).load(date_for=date_for)
+        assert cdm.average_progress == expected_val
+
     @pytest.mark.skip('Implement me!')
     def test_load_force_update(self):
         pass
