@@ -1,3 +1,4 @@
+from __future__ import absolute_import
 import calendar
 import datetime
 from django.utils.timezone import utc
@@ -20,9 +21,13 @@ from figures.helpers import (
     prev_day,
     previous_months_iterator,
     first_last_days_for_month,
+    import_from_path,
+    utc_yesterday,
     )
 
 from tests.factories import COURSE_ID_STR_TEMPLATE
+import six
+from six.moves import range
 
 
 class TestCourseKeyHelper(object):
@@ -83,9 +88,9 @@ class TestDateTimeHelper(object):
 
     def test_get_now_from_unicode(self):
         format = '%Y-%m-%d %H:%M:%S'
-        a_datetime_str = unicode(self.now.strftime(format))
+        a_datetime_str = six.text_type(self.now.strftime(format))
         expected = dateutil_parse(a_datetime_str).replace(tzinfo=utc)
-        assert isinstance(a_datetime_str, unicode)
+        assert isinstance(a_datetime_str, six.text_type)
         assert as_datetime(a_datetime_str) == expected
 
     def test_get_now_from_date(self):
@@ -127,9 +132,9 @@ class TestDateHelper(object):
 
     def test_get_now_from_unicode(self):
         format = '%Y-%m-%d'
-        a_date_str = unicode(self.now.strftime(format))
+        a_date_str = six.text_type(self.now.strftime(format))
         expected = self.now.date()
-        assert isinstance(a_date_str, unicode)
+        assert isinstance(a_date_str, six.text_type)
         assert as_date(a_date_str) == expected
 
     def test_get_now_from_date(self):
@@ -157,7 +162,7 @@ class TestDeltaDays(object):
     def setup(self):
         self.now = datetime.datetime(2018, 6, 1)
 
-    @pytest.mark.parametrize('days', range(-2, 3))
+    @pytest.mark.parametrize('days', list(range(-2, 3)))
     def test_days_from(self, days):
         '''TODO: Test with input as a
         - datetime
@@ -185,23 +190,20 @@ class TestDeltaDays(object):
 
 class TestMonthIterator(object):
 
-    @pytest.mark.parametrize('month_for, months_back, first_month', [
-        ((2018, 1, 31), 0, datetime.date(2018, 1, 1)),
-        ((2018, 1, 31), 6, datetime.date(2017, 7, 1)),
-        ])
-    def test_previous_months_iterator(self, month_for, months_back, first_month):
-
-        def as_month_tuple(month):
-            return (month.year, month.month)
+    @pytest.mark.parametrize('month_for, months_back', [
+        ((2018, 1, 31), 0),
+        ((2018, 1, 31), 6),
+        ((2020, 4, 30), 2),  # let's use a leap year
+    ])
+    def test_previous_months_iterator(self, month_for, months_back):
         expected_vals = []
-        for i in range(months_back):
-            a_month = first_month+relativedelta(months=i)
+        month_for = datetime.date(year=month_for[0], month=month_for[1], day=month_for[2])
+        for i in range(max(1, months_back), 0, -1):  # e.g., 6, 5, 4, 3, 2, 1
+            a_month = month_for - relativedelta(months=i - 1)
             last_day_in_month = calendar.monthrange(a_month.year, a_month.month)[1]
             expected_vals.append(
                 (a_month.year, a_month.month, last_day_in_month)
-                )
-        expected_vals.append(month_for)
-
+            )
         vals = list(previous_months_iterator(month_for, months_back))
         assert vals == expected_vals
 
@@ -217,3 +219,42 @@ def test_first_last_days_for_month():
     assert last_day.year == year
     assert first_day.day == 1
     assert last_day.day == 29
+
+
+def test_import_from_path_working():
+    utc_tz_path = 'django.utils.timezone:utc'
+    imported_utc = import_from_path(utc_tz_path)
+    assert imported_utc is utc, 'Should import the utc variable correctly'
+
+
+def test_import_from_path_failing():
+    utc_tz_path = 'django.utils.timezone:non_existent_variable'
+    with pytest.raises(AttributeError):
+        # Should raise an AttributeError because the helper is using getattr()
+        import_from_path(utc_tz_path)
+
+
+def test_import_from_path_missing_module():
+    utc_tz_path = 'non_existent_module.submodule:some_variable'
+    with pytest.raises(ImportError):
+        # Should raise an ImportError because the module does not exist
+        import_from_path(utc_tz_path)
+
+
+def test_import_from_bad_syntax():
+    utc_tz_path = 'not a path'
+    with pytest.raises(ValueError):
+        # Malformed path
+        import_from_path(utc_tz_path)
+
+
+def test_utc_yesterday():
+    """Basic sanity check and test coverage using live time
+
+    If we find a need or wante to get elaborate, we test a specific time for
+    any time by with monkeypatch/mock of `utcnow()` call in `figures.helpers`
+    If it is for a need because the expected and actual do not match up, then
+    document why.
+    """
+    expected = datetime.datetime.utcnow().date() - datetime.timedelta(days=1)
+    assert utc_yesterday() == expected
