@@ -2,20 +2,19 @@
 
 '''
 
+from __future__ import absolute_import
 import datetime
 from dateutil.parser import parse as dateutil_parse
 from decimal import Decimal
-from dateutil.parser import parse
 import pytest
-import pytz
+# import pytz
 
 from django.contrib.sites.models import Site
 from django.db import models
 from django.utils.timezone import utc
 from rest_framework.exceptions import ValidationError
 
-from student.models import CourseEnrollment
-
+from figures.compat import CourseEnrollment
 from figures.models import (
     CourseDailyMetrics,
     CourseMauMetrics,
@@ -29,6 +28,7 @@ from figures.serializers import (
     CourseEnrollmentSerializer,
     CourseMauMetricsSerializer,
     CourseMauLiveMetricsSerializer,
+    EnrollmentDataSerializer,
     GeneralCourseDataSerializer,
     GeneralUserDataSerializer,
     LearnerCourseDetailsSerializer,
@@ -46,6 +46,7 @@ from tests.factories import (
     CourseEnrollmentFactory,
     CourseMauMetricsFactory,
     CourseOverviewFactory,
+    EnrollmentDataFactory,
     GeneratedCertificateFactory,
     LearnerCourseGradeMetricsFactory,
     SiteDailyMetricsFactory,
@@ -54,7 +55,8 @@ from tests.factories import (
     SiteFactory,
     )
 
-from tests.helpers import platform_release
+from tests.helpers import as_datetime_utc, platform_release
+import six
 
 
 class TestSerializableCountryField(object):
@@ -145,8 +147,8 @@ class TestCourseDetailsSerializer(object):
         assert data['course_name'] == self.course_overview.display_name
         assert data['course_code'] == self.course_overview.number
         assert data['org'] == self.course_overview.org
-        assert parse(data['start_date']) == self.course_overview.start
-        assert parse(data['end_date']) == self.course_overview.end
+        assert as_datetime_utc(data['start_date']) == self.course_overview.start
+        assert as_datetime_utc(data['end_date']) == self.course_overview.end
         assert data['self_paced'] == self.course_overview.self_paced
 
     def test_get_staff_with_no_course(self):
@@ -184,7 +186,7 @@ class TestCourseEnrollmentSerializer(object):
         # assert data['course']['display_name'] == self.model_obj.course.display_name
         # assert data['course']['org'] == self.model_obj.course.org
 
-        assert dateutil_parse(data['created']) == self.model_obj.created
+        assert as_datetime_utc(data['created']) == self.model_obj.created
         assert data['user']['fullname'] == self.model_obj.user.profile.name
 
         for field_name in (self.expected_results_keys - self.special_fields):
@@ -231,8 +233,8 @@ class TestCourseDailyMetricsSerializer(object):
 
         # Hack: Check date and datetime values explicitly
         assert data['date_for'] == str(self.metrics.date_for)
-        assert dateutil_parse(data['created']) == self.metrics.created
-        assert dateutil_parse(data['modified']) == self.metrics.modified
+        assert as_datetime_utc(data['created']) == self.metrics.created
+        assert as_datetime_utc(data['modified']) == self.metrics.modified
         check_fields = self.expected_results_keys - self.date_fields - set(['site'])
         for field_name in check_fields:
             db_field = getattr(self.metrics, field_name)
@@ -247,7 +249,7 @@ class TestCourseDailyMetricsSerializer(object):
         serializer = CourseDailyMetricsSerializer(instance=obj)
         check_val = Decimal(average_progress).quantize(Decimal('.00'))
         data = serializer.data
-        assert data['average_progress'] == unicode(check_val)
+        assert data['average_progress'] == six.text_type(check_val)
 
     @pytest.mark.xfail
     @pytest.mark.parametrize('average_progress', [-1.0, 9.0])
@@ -294,8 +296,8 @@ class TestSiteDailyMetricsSerializer(object):
 
         # Hack: Check date and datetime values explicitly
         assert data['date_for'] == str(self.site_daily_metrics.date_for)
-        assert dateutil_parse(data['created']) == self.site_daily_metrics.created
-        assert dateutil_parse(data['modified']) == self.site_daily_metrics.modified
+        assert as_datetime_utc(data['created']) == self.site_daily_metrics.created
+        assert as_datetime_utc(data['modified']) == self.site_daily_metrics.modified
         check_fields = self.expected_results_keys - self.date_fields - set(['site'])
         for field_name in check_fields:
             assert data[field_name] == getattr(self.site_daily_metrics,field_name)
@@ -321,6 +323,29 @@ class TestSiteDailyMetricsSerializer(object):
         assert SiteDailyMetrics.objects.count() == 2
 
 
+@pytest.mark.django_db
+class TestEnrollmentDataSerializer(object):
+    @pytest.fixture(autouse=True)
+    def setup(self, db):
+        self.enrollment_data = EnrollmentDataFactory()
+        self.expected_fields = [
+            'id', 'course_id', 'date_enrolled', 'is_enrolled', 'is_completed',
+            'progress_percent', 'progress_details',
+        ]
+        self.date_enrolled_format = '%Y-%m-%d'
+
+    def test_serialize(self):
+        """Test Serializing EnrollmentData model with EnrollmentDataSerializer
+
+        Minimal test after bug fix for https://github.com/appsembler/figures/issues/364
+        """
+        expected_date_enrolled = self.enrollment_data.date_enrolled.strftime(
+            self.date_enrolled_format)
+        serializer = EnrollmentDataSerializer(self.enrollment_data)
+        assert set(serializer.data.keys()) == set(self.expected_fields)
+        assert serializer.data['date_enrolled'] == expected_date_enrolled
+
+
 @pytest.mark.django_db
 class TestGeneralCourseDataSerializer(object):
     '''
@@ -359,8 +384,8 @@ class TestGeneralCourseDataSerializer(object):
         assert data['course_name'] == self.course_overview.display_name
         assert data['course_code'] == self.course_overview.number
         assert data['org'] == self.course_overview.org
-        assert parse(data['start_date']) == self.course_overview.start
-        assert parse(data['end_date']) == self.course_overview.end
+        assert as_datetime_utc(data['start_date']) == self.course_overview.start
+        assert as_datetime_utc(data['end_date']) == self.course_overview.end
         assert data['self_paced'] == self.course_overview.self_paced
 
     def test_get_metrics_with_cdm_records(self):
@@ -384,7 +409,7 @@ class TestGeneralUserDataSerializer(object):
 
     @pytest.fixture(autouse=True)
     def setup(self, db):
-        self.a_datetime = datetime.datetime(2018, 2, 2, tzinfo=pytz.UTC)
+        self.a_datetime = datetime.datetime(2018, 2, 2, tzinfo=utc)
         self.user_attributes = {
             'username': 'alpha_one',
             'email': 'alpha_one@example.com',
@@ -602,7 +627,7 @@ class TestCourseMauMetricsSerializer(object):
         assert data['mau'] == self.obj.mau
         assert data['domain'] == self.obj.site.domain
         assert data['course_id'] == self.obj.course_id
-        assert dateutil_parse(data['date_for']).date() == self.obj.date_for 
+        assert dateutil_parse(data['date_for']).date() == self.obj.date_for
 
 
 @pytest.mark.django_db
