@@ -22,6 +22,7 @@ or restructure into fixtures and standalone test functions, depending on how
 figures.sites evolves
 """
 
+from __future__ import absolute_import
 import mock
 import pytest
 
@@ -30,10 +31,7 @@ from django.contrib.sites.models import Site
 
 import organizations
 
-from openedx.core.djangoapps.content.course_overviews.models import (
-    CourseOverview,
-)
-
+from figures.compat import CourseOverview
 import figures.helpers
 import figures.sites
 
@@ -47,6 +45,7 @@ from tests.factories import (
     UserFactory,
 )
 from tests.helpers import organizations_support_sites
+from six.moves import range
 
 
 if organizations_support_sites():
@@ -135,7 +134,9 @@ class TestHandlersForMultisiteMode(object):
         is_multisite = figures.helpers.is_multisite()
         assert is_multisite
         self.site = SiteFactory(domain='foo.test')
+        self.default_site = Site.objects.get(id=1)
         self.organization = OrganizationFactory(sites=[self.site])
+        self.default_site_org = OrganizationFactory(sites=[self.default_site])
         assert Site.objects.count() == 2
         self.features = {'FIGURES_IS_MULTISITE': True}
 
@@ -194,16 +195,41 @@ class TestHandlersForMultisiteMode(object):
 
     @pytest.mark.parametrize('ce_count', [0, 1, 2])
     def test_get_course_enrollments_for_site(self, ce_count):
-
         course_overview = CourseOverviewFactory()
         OrganizationCourseFactory(organization=self.organization,
                                   course_id=str(course_overview.id))
+        uoms = [UserOrganizationMappingFactory(
+            organization=self.organization) for i in range(ce_count)]
+
         expected_ce = [CourseEnrollmentFactory(
-            course_id=course_overview.id) for i in range(ce_count)]
+            course_id=course_overview.id,
+            user=uoms[i].user) for i in range(ce_count)]
         course_enrollments = figures.sites.get_course_enrollments_for_site(self.site)
         assert set([ce.id for ce in course_enrollments]) == set(
                    [ce.id for ce in expected_ce])
 
+    def test_get_course_enrollments_for_site_exclude_same_user_different_site(self):
+        """
+        Test that CEs are not returned from course from another Site, in cases where a user has
+        CEs in desired Site, but also in another Site.
+        """
+        course_overviews = [CourseOverviewFactory() for i in range(2)]
+        OrganizationCourseFactory(organization=self.organization,
+                                  course_id=str(course_overviews[0].id))
+        OrganizationCourseFactory(organization=self.default_site_org,
+                                  course_id=str(course_overviews[1].id))
+        uom_our_site = UserOrganizationMappingFactory(organization=self.organization)
+
+        # enroll same user in a course associated w/ an Organization not connected to our Site
+        uom_other_site = UserOrganizationMappingFactory(user=uom_our_site.user, organization=self.default_site_org)
+        CourseEnrollmentFactory(course_id=course_overviews[1].id, user=uom_our_site.user)
+
+        expected_ce = [CourseEnrollmentFactory(course_id=course_overviews[0].id, user=uom_our_site.user)]
+        course_enrollments = figures.sites.get_course_enrollments_for_site(self.site)
+        assert set([ce.id for ce in course_enrollments]) == set(
+                   [ce.id for ce in expected_ce])
+
+
     def test_get_student_modules_for_course_in_site(self):
         course_overviews = [CourseOverviewFactory() for i in range(3)]
 
@@ -331,3 +357,170 @@ def test_site_iterator():
         collected_ids.append(site_id)
 
     assert set(collected_ids) == set([site.id for site in sites])
+
+
+@pytest.mark.django_db
+@pytest.fixture
+def enrollment_data(db):
+    """Test data for course id filtering
+    """
+    course_overviews = [CourseOverviewFactory() for i in range(2)]
+    expected_enrollments = []
+    for co in course_overviews:
+        expected_enrollments += [CourseEnrollmentFactory(course_id=co.id)
+                                 for i in range(2)]
+    # Create enrollment we don't want
+    other_enrollment = CourseEnrollmentFactory()
+    return dict(
+        course_overviews=course_overviews,
+        expected_enrollments=expected_enrollments,
+        other_enrollment=other_enrollment)
+
+
+def test_enrollments_for_course_ids(enrollment_data):
+    course_ids = [co.id for co in enrollment_data['course_overviews']]
+    expected_enrollments = enrollment_data['expected_enrollments']
+    enrollments = figures.sites.enrollments_for_course_ids(course_ids)
+    assert set(enrollments) == set(expected_enrollments)
+
+
+def test_users_enrolled_in_courses(enrollment_data):
+    course_ids = [co.id for co in enrollment_data['course_overviews']]
+    expected_enrollments = enrollment_data['expected_enrollments']
+    expected_users = [ce.user for ce in expected_enrollments]
+    users = figures.sites.users_enrolled_in_courses(course_ids)
+    assert set(users) == set(expected_users)
+
+
+@pytest.mark.django_db
+def test_site_course_ids(monkeypatch):
+    site = SiteFactory()
+    course_overviews = [CourseOverviewFactory() for i in range(2)]
+    if organizations_support_sites():
+        monkeypatch.setattr('figures.sites.is_multisite', lambda: True)
+        our_org = OrganizationFactory(sites=[site])
+        # associate the course overviews with our org
+        for co in course_overviews:
+            OrganizationCourseFactory(course_id=co.id, organization=our_org)
+        other_org = OrganizationFactory(sites=[SiteFactory()])
+        # create a course associated with another org
+        co = CourseOverviewFactory()
+        OrganizationCourseFactory(course_id=co.id, organization=other_org)
+        
+    course_ids = figures.sites.site_course_ids(site)
+    assert set(course_ids) == set([str(co.id) for co in course_overviews])
+
+
+@pytest.mark.django_db
+def test_student_modules_for_course_enrollment(monkeypatch):
+    """Test we get the correct student modules for the given course enrollment
+    """
+    site = SiteFactory()
+    ce = CourseEnrollmentFactory()
+    ce_sm = [StudentModuleFactory(student=ce.user, course_id=ce.course_id)]
+    # Create another student module record to make sure this is not in our
+    # query results
+    StudentModuleFactory()
+
+    if organizations_support_sites():
+        monkeypatch.setattr('figures.sites.is_multisite', lambda: True)
+        our_org = OrganizationFactory(sites=[site])
+        other_org = OrganizationFactory(sites=[SiteFactory()])
+        other_org_ce = CourseEnrollmentFactory()
+        other_sm = StudentModuleFactory(student=other_org_ce.user,
+                                        course_id=other_org_ce.course_id)
+        UserOrganizationMappingFactory(user=ce.user,organization=our_org)
+        UserOrganizationMappingFactory(user=other_org_ce.user,
+                                       organization=other_org)
+
+    sm = figures.sites.student_modules_for_course_enrollment(site, ce)
+    assert set(sm) == set(ce_sm)
+
+
+@pytest.mark.django_db
+def test_get_requested_site_default_behaviour(settings):
+    """
+    Test `get_requested_site` returns Django's get_current_site() by default.
+    """
+    example_site = Site.objects.get()  # gets the example site
+    settings.SITE_ID = example_site.id
+
+    current_site = figures.sites.get_requested_site(request=mock.Mock())
+    assert current_site == example_site, 'Use Django\'s get_current_site().'
+
+
+@pytest.mark.django_db
+def test_get_requested_site_custom_backend(settings):
+    """
+    Test `get_requested_site` can use custom backends.
+    """
+    orange_site = SiteFactory.create(name='orange site')
+
+    settings.ENV_TOKENS = {
+        'FIGURES': {
+            'REQUESTED_SITE_BACKEND': 'organizations:get_orange_site'
+        }
+    }
+    with mock.patch('organizations.get_orange_site', create=True, return_value=orange_site):
+        requested_site = figures.sites.get_requested_site(request=mock.Mock())
+    assert requested_site == orange_site, 'Should use custom backend.'
+
+
+@pytest.mark.django_db
+def test_get_requested_site_broken_backend(settings):
+    """
+    Test `get_requested_site` don't hide errors from custom backends.
+
+    Figures should keep a simple backend implementation without attempting to fix errors in site configuration or
+    faulty backends.
+    """
+    settings.ENV_TOKENS = {
+        'FIGURES': {
+            'REQUESTED_SITE_BACKEND': 'organizations:broken_backend'
+        }
+    }
+    with mock.patch('organizations.broken_backend', create=True, side_effect=RuntimeError):
+        with pytest.raises(RuntimeError):
+            # Should fail if the REQUESTED_SITE_BACKEND fails
+            figures.sites.get_requested_site(request=mock.Mock())
+
+
+@pytest.mark.skipif(not organizations_support_sites(), reason='needed only in multisite mode')
+@pytest.mark.django_db
+def test_get_sites_default_behaviour():
+    default_site = Site.objects.get()  # gets the example site
+    another_site = SiteFactory()
+    all_sites = figures.sites.get_sites()
+    assert list(all_sites) == [default_site, another_site], 'Should return all sites.'
+
+
+@pytest.mark.skipif(not organizations_support_sites(), reason='needed only in multisite mode')
+@pytest.mark.django_db
+def test_get_sites_custom_backend(settings):
+    _orange_site = SiteFactory(name='orange site')
+    blue_site_1 = SiteFactory(name='blue site 1')
+    blue_site_2 = SiteFactory(name='blue site 2')
+
+    blue_sites = Site.objects.filter(name__startswith='blue site')
+
+    settings.ENV_TOKENS = {
+        'FIGURES': {
+            'SITES_BACKEND': 'organizations:get_blue_sites'
+        }
+    }
+    with mock.patch('organizations.get_blue_sites', create=True, return_value=blue_sites):
+        all_sites = figures.sites.get_sites()
+    assert list(all_sites) == [blue_site_1, blue_site_2], 'Should return just blue sites.'
+
+
+@pytest.mark.skipif(not organizations_support_sites(), reason='needed only in multisite mode')
+@pytest.mark.django_db
+def test_get_sites_broken_backend(settings):
+    settings.ENV_TOKENS = {
+        'FIGURES': {
+            'SITES_BACKEND': 'organizations:broken_backend'
+        }
+    }
+    with mock.patch('organizations.broken_backend', create=True, side_effect=ValueError):
+        with pytest.raises(ValueError):
+            figures.sites.get_sites()  # Should fail if the SITES_BACKEND fails
